package cas.cordova.plugin;

import android.app.Activity;
import android.content.pm.PackageManager;
import android.util.Log;

import com.cleversolutions.ads.AdLoadCallback;
import com.cleversolutions.ads.AdNetwork;
import com.cleversolutions.ads.AdType;
import com.cleversolutions.ads.ConsentFlow;
import com.cleversolutions.ads.LastPageAdContent;
import com.cleversolutions.ads.MediationManager;
import com.cleversolutions.ads.android.CAS;
import com.cleveradssolutions.sdk.base.CASHandler;
import com.google.gson.Gson;

import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaInterface;
import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.CordovaWebView;
import org.jetbrains.annotations.NotNull;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;

import static cas.cordova.plugin.Constants.AD_TYPE_BANNER;
import static cas.cordova.plugin.Constants.AD_TYPE_INTER;
import static cas.cordova.plugin.Constants.AD_TYPE_RETURN;
import static cas.cordova.plugin.Constants.AD_TYPE_REWARD;


/**
 * This class echoes a string called from JavaScript.
 */
public class CASCBridge extends CordovaPlugin {
    //Properties
    private boolean isReturnAds = false;

    //CAS
    private MediationManager manager;

    private cas.cordova.plugin.BannerWrapper banner;

    //Listeners
    private final CASBannerCallbackWrapper bannerListener = new CASBannerCallbackWrapper(AdType.Banner.ordinal(), this);
    private final CASCallbackWrapper interstitialListener = new CASCallbackWrapper(AdType.Interstitial.ordinal(), this);
    private final CASCallbackWrapper rewardedListener = new CASCallbackWrapper(AdType.Rewarded.ordinal(), this);
    private final CASCallbackWrapper appReturnListener = new CASCallbackWrapper(AD_TYPE_RETURN, this);

    // ------ Utilities :: Cordova
    private CordovaWebView webview;

    @Override
    public void initialize(final CordovaInterface cordova, final CordovaWebView webview) {
        this.webview = webview;
        super.initialize(cordova, webview);
    }

    @Override
    public boolean execute(String action, JSONArray args, CallbackContext callbackContext) {
        cordova.getActivity().runOnUiThread(() -> {
            switch (action) {
                case "initialize":
                    initialize(args, callbackContext);
                    break;
                case "initializeWithDefaultParameters":
                    initializeWithDefaultParameters(args, callbackContext);
                case "validateIntegration":
                    validateIntegration(callbackContext);
                    break;
                case "skipNextReturnAds":
                    skipNextReturnAds(callbackContext);
                    break;
                case "loadAd":
                    loadAd(args, callbackContext);
                    break;
                case "showAd":
                    showAd(args, callbackContext);
                    break;
                case "hideBanner":
                    hideBanner(callbackContext);
                    break;
                case "isAdReady":
                    isAdReady(args, callbackContext);
                    break;
                case "isEnabled":
                    isEnabled(args, callbackContext);
                    break;
                case "enableAd":
                    enableAd(args, callbackContext);
                    break;
                case "setBannerSizeId":
                    setBannerSizeId(args, callbackContext);
                    break;
                case "setBannerPositionId":
                    setBannerPositionId(args, callbackContext);
                    break;
                case "setLastPageAdContent":
                    setLastPageAdContent(args, callbackContext);
                    break;
                case "setDebugMode":
                    setDebugMode(args, callbackContext);
                    break;
                case "setBannerInSafeArea":
                    setBannerInSafeArea(args, callbackContext);
                    break;
                case "getSDKVersion":
                    getSDKVersion(callbackContext);
                    break;
                case "setUserConsentStatus":
                    setUserConsentStatus(args, callbackContext);
                    break;
                case "setDoNotSellStatus":
                    setDoNotSellStatus(args, callbackContext);
                    break;
                case "setTaggedAudience":
                    setTaggedAudience(args, callbackContext);
                    break;
                case "setMutedAdSounds":
                    setMutedAdSounds(args, callbackContext);
                    break;
                case "setLoadingMode":
                    setLoadingMode(args, callbackContext);
                    break;
                case "setRefreshBannerDelay":
                    setRefreshBannerDelay(args, callbackContext);
                    break;
                case "setInterstitialInterval":
                    setInterstitialInterval(args, callbackContext);
                    break;
                case "restartInterstitialInterval":
                    restartInterstitialInterval(callbackContext);
                    break;
                case "setAnalyticsCollectionEnabled":
                    setAnalyticsCollectionEnabled(args, callbackContext);
                    break;
                case "setTestDeviceIds":
                    setTestDeviceIds(args, callbackContext);
                    break;
                case "setUserGender":
                    setUserGender(args, callbackContext);
                    break;
                case "setUserAge":
                    setUserAge(args, callbackContext);
                    break;
                case "getActiveMediationPattern":
                    getActiveMediationPattern(callbackContext);
                    break;
                case "isActiveMediationNetwork":
                    isActiveMediationNetwork(args, callbackContext);
                    break;
                case "setAllowInterstitialAdsWhenRewardedCostAreLower":
                    setAllowInterstitialAdsWhenRewardedCostAreLower(args, callbackContext);
                    break;
                default:
                    callbackContext.error("\"" + action + "\" is not a recognized \"CASCordova\" method.");
            }
        });

        return true;
    }

    private void initialize(final JSONArray args, final CallbackContext callbackContext) {
        try {
            final JSONObject mode = args.getJSONObject(0);

            String managerId = mode.optString("managerId", "demo");
            boolean isTestMode = mode.optBoolean("testMode", true);
            int enableAdTypes = mode.optInt("enableAdTypes", 0);
            isReturnAds = mode.optBoolean("returnAds", false);
            String platform = mode.optString("frameworkName", "");
            String version = mode.optString("frameworkVersion", "");
            boolean isConsent = mode.optBoolean("consent", true);
            String privacyPolicy = mode.optString("privacyPolicy", "");

            initialize(managerId, isTestMode, enableAdTypes, platform, version, isConsent, privacyPolicy, callbackContext);
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("initialization", e.toString()));
        }
    }

    private void initializeWithDefaultParameters(final JSONArray args, final CallbackContext callbackContext) {
        try {
            final JSONObject mode = args.getJSONObject(0);
            int enableAdTypes = mode.optInt("enableAdTypes", 0);
            String managerId = mode.optString("managerId", "demo");
            boolean isTestMode = mode.optBoolean("testMode", true);

            isReturnAds = mode.optBoolean("returnAds", false);
            boolean isDebugMode = mode.optBoolean("debugMode", false);
            boolean isAllowInterstitialAdsWhenRewardedCostAreLower = mode.optBoolean("isAllowInterstitialAdsWhenRewardedCostAreLower", false);
            boolean isValidateIntegration = mode.optBoolean("validateIntegration", false);
            boolean isBannerInSafeArea = mode.optBoolean("bannerInSafeArea", true);
            int GDPR = mode.optInt("GDPR", 0);
            int CCPA = mode.optInt("CCPA", 0);
            int taggedAudience = mode.optInt("taggedAudience", 0);
            int loadingMode = mode.optInt("loadingMode", 2);
            int interstitialInterval = mode.optInt("interstitialInterval", 0);
            int refreshBannerDelay = mode.optInt("refreshBannerDelay", 30);
            boolean analyticsCollectionEnable = mode.optBoolean("analyticsCollectionEnable", false);
            String platform = mode.optString("frameworkName", "");
            String version = mode.optString("frameworkVersion", "");

            initialize(managerId, isTestMode, enableAdTypes, platform, version, true, "", callbackContext);

            CAS.getSettings().setDebugMode(isDebugMode);
            CAS.getSettings().setAllowInterstitialAdsWhenVideoCostAreLower(isAllowInterstitialAdsWhenRewardedCostAreLower);

            if (isValidateIntegration)
                CAS.validateIntegration(cordova.getActivity());

            banner.isNeedSafeInsets = isBannerInSafeArea;

            CAS.getSettings().setUserConsent(GDPR);
            CAS.getSettings().setCcpaStatus(CCPA);
            CAS.getSettings().setTaggedAudience(taggedAudience);
            CAS.getSettings().setLoadingMode(loadingMode);
            CAS.getSettings().setInterstitialInterval(interstitialInterval);
            CAS.getSettings().setBannerRefreshInterval(refreshBannerDelay);
            CAS.getSettings().setAnalyticsCollectionEnabled(analyticsCollectionEnable);
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("initializeWithDefaultParameters", e.toString()));
        }
    }

    private void initialize(String managerId,
                            boolean isTestMode,
                            int enableAdTypes,
                            String platform,
                            String version,
                            boolean isConsent,
                            String url,
                            CallbackContext callbackContext) {

        ConsentFlow flow = new ConsentFlow(isConsent);
        if (isConsent && !url.isEmpty())
            flow.setPrivacyPolicyUrl(url);

        CAS.ManagerBuilder builder = CAS.buildManager()
                .withManagerId(managerId)
                .withCompletionListener((config) -> {
                    boolean successful = config.getError() == null;
                    if (successful)
                        callbackContext.success();
                    else {
                        Log.e("CASInit", config.getError());
                        callbackContext.error(config.getError());
                    }
                })
                .withConsentFlow(flow)
                .withEnabledAdTypes(enableAdTypes)
                .withFramework(platform, version)
                .withTestAdMode(isTestMode);

        Activity activity = cordova.getActivity();

        manager = builder.initialize(activity);

        if (isReturnAds) {
            manager.enableAppReturnAds(appReturnListener);
        }

        manager.getOnAdLoadEvent().add(new AdLoadCallback() {
            @Override
            public void onAdLoaded(@NotNull AdType adType) {
                fireEventTrigger(adType.ordinal() + "_Loaded");
            }

            @Override
            public void onAdFailedToLoad(@NotNull AdType adType, @org.jetbrains.annotations.Nullable String s) {
                fireEventTrigger(adType.ordinal() + "_LoadFailed");
                Log.e("CAS", adType.toString() + " failed to load");
            }
        });

        if (banner == null)
            banner = new BannerWrapper(manager, activity, bannerListener, this);
    }

    private void setDebugMode(final JSONArray args, final CallbackContext callbackContext) {
        try {
            CAS.getSettings().setDebugMode(args.getBoolean(0));
            callbackContext.success();
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("setDebugMode", e.toString()));
        }
    }

    private void skipNextReturnAds(final CallbackContext callbackContext) {
        try {
            manager.skipNextAppReturnAds();
            callbackContext.success();
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("skipNextReturnAds", e.toString()));
        }
    }

    private void loadAd(final JSONArray args, final CallbackContext callbackContext) {
        try {
            int adType = args.getInt(0);

            if (adType == AD_TYPE_BANNER) {
                banner.loadNextAd();
                callbackContext.success();
            } else if (adType == AD_TYPE_INTER) {
                manager.loadInterstitial();
                callbackContext.success();
            } else if (adType == AD_TYPE_REWARD) {
                manager.loadRewardedAd();
                callbackContext.success();
            } else {
                callbackContext.error("Cordova bridge call load ad for unknown type id: " + adType);
            }

        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("loadAd", e.toString()));
        }
    }

    private void showAd(final JSONArray args, final CallbackContext callbackContext) {
        try {
            int adType = args.getInt(0);

            if (adType == AD_TYPE_BANNER) {
                CASHandler.INSTANCE.main(banner.showRunnable);
                callbackContext.success();
            } else if (adType == AD_TYPE_INTER) {
                manager.showInterstitial(cordova.getActivity(), interstitialListener);
                callbackContext.success();
            } else if (adType == AD_TYPE_REWARD) {
                manager.showRewardedAd(cordova.getActivity(), rewardedListener);
                callbackContext.success();
            } else {
                callbackContext.error("Cordova java bridge call load ad for unknown type id: " + adType);
            }

        } catch (JSONException e) {
            callbackContext.error(debugErrorLog("loadAd", e.toString()));
        }
    }

    private void hideBanner(CallbackContext callbackContext) {
        try {
            CASHandler.INSTANCE.main(banner.hideRunnable);
            callbackContext.success();
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("hideBanner", e.toString()));
        }
    }

    private void isAdReady(final JSONArray args, final CallbackContext callbackContext) {
        try {
            AdType adType = getAdType(args.getInt(0));

            if (adType == AdType.None) {
                callbackContext.error("Cordova java bridge call isAdReady for unknown type id: " + adType);
                return;
            }

            boolean isReady = false;
            if (adType == AdType.Banner)
                isReady = true;
            else if (adType == AdType.Interstitial)
                isReady = manager.isInterstitialReady();
            else if (adType == AdType.Rewarded)
                isReady = manager.isRewardedAdReady();

            final String readyKey = "isReady";

            final JSONObject data = new JSONObject();
            data.put(readyKey, isReady);

            callbackContext.success(data);
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("isAdReady", e.toString()));
        }
    }


    private void isEnabled(final JSONArray args, final CallbackContext callbackContext) {
        try {
            AdType adType = getAdType(args.getInt(0));

            if (adType == AdType.None) {
                callbackContext.error("Cordova java bridge call isEnabled for unknown type id: " + adType);
                return;
            }

            final boolean isEnable = manager.isEnabled(adType);
            final String isEnableKey = "isEnable";

            final JSONObject data = new JSONObject();
            data.put(isEnableKey, isEnable);

            callbackContext.success(data);
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("isEnabled", e.toString()));
        }
    }

    private void enableAd(final JSONArray args, final CallbackContext callbackContext) {
        try {
            int type = args.getInt(0);
            boolean enable = args.getBoolean(1);
            AdType adType = getAdType(type);

            if (adType == AdType.None) {
                callbackContext.error("Cordova java bridge call enableAd for unknown type id: " + adType);
                return;
            }

            manager.setEnabled(adType, enable);
            callbackContext.success();
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("enableAd", e.toString()));
        }
    }

    private void setBannerSizeId(final JSONArray args, final CallbackContext callbackContext) {
        try {
            int targetSize = args.getInt(0);
            if (!banner.setBannerSizeId(targetSize)) {
                callbackContext.error("Cordova java bridge call change banner size with unknown id: " + targetSize);
                return;
            }
            callbackContext.success();
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("setBannerSizeId", e.toString()));
        }
    }

    private void setBannerPositionId(final JSONArray args, final CallbackContext callbackContext) {
        try {
            banner.setBannerPositionId(args.getInt(0));
            callbackContext.success();
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("setBannerPositionId", e.toString()));
        }
    }

    private void validateIntegration(CallbackContext callbackContext) {
        try {
            CAS.validateIntegration(cordova.getActivity());
            callbackContext.success();
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("validateIntegration", e.toString()));
        }
    }

    private void setLastPageAdContent(final JSONArray args, final CallbackContext callbackContext) {
        try {
            String json = args.getString(0);

            if (json == null || json.length() == 0) {
                manager.setLastPageAdContent(null);
                callbackContext.success();
                return;
            }

            try {
                manager.setLastPageAdContent(new Gson().fromJson(json, LastPageAdContent.class));
                callbackContext.success();
            } catch (Throwable var) {
                callbackContext.error("Cordova java bridge set Last Page Ad Content error " + var.getLocalizedMessage());
            }
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("setBannerSizeId", e.toString()));
        }
    }

    private void setBannerInSafeArea(final JSONArray args, final CallbackContext callbackContext) {
        try {
            banner.isNeedSafeInsets = args.getBoolean(0);
            callbackContext.success();
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("setBannerInSafeArea", e.toString()));
        }
    }

    //CAS Settings
    private void getSDKVersion(final CallbackContext callbackContext) {
        try {
            final JSONObject data = new JSONObject();
            data.put("sdkVersion", CAS.getSDKVersion());
            callbackContext.success(data);
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("getSDKVersion", e.toString()));
        }
    }

    private void setUserConsentStatus(final JSONArray args, final CallbackContext callbackContext) {
        try {
            CAS.getSettings().setUserConsent(args.getInt(0));
            callbackContext.success();
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("setUserConsentStatus", e.toString()));
        }
    }

    private void setDoNotSellStatus(final JSONArray args, final CallbackContext callbackContext) {
        try {
            CAS.getSettings().setCcpaStatus(args.getInt(0));
            callbackContext.success();
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("setDoNotSellStatus", e.toString()));
        }
    }

    private void setTaggedAudience(final JSONArray args, final CallbackContext callbackContext) {
        try {
            CAS.getSettings().setTaggedAudience(args.getInt(0));
            callbackContext.success();
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("setTaggedAudience", e.toString()));
        }
    }

    private void setMutedAdSounds(final JSONArray args, final CallbackContext callbackContext) {
        try {
            CAS.getSettings().setMutedAdSounds(args.getBoolean(0));
            callbackContext.success();
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("setMutedAdSounds", e.toString()));
        }
    }

    private void setLoadingMode(final JSONArray args, final CallbackContext callbackContext) {
        try {
            CAS.getSettings().setLoadingMode(args.getInt(0));
            callbackContext.success();
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("setLoadingMode", e.toString()));
        }
    }

    private void setRefreshBannerDelay(final JSONArray args, final CallbackContext callbackContext) {
        try {
            CAS.getSettings().setBannerRefreshInterval(args.getInt(0));
            callbackContext.success();
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("setRefreshBannerDelay", e.toString()));
        }
    }

    private void setInterstitialInterval(final JSONArray args, final CallbackContext callbackContext) {
        try {
            CAS.getSettings().setInterstitialInterval(args.getInt(0));
            callbackContext.success();
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("setInterstitialInterval", e.toString()));
        }
    }

    private void restartInterstitialInterval(final CallbackContext callbackContext) {
        try {
            CAS.getSettings().restartInterstitialInterval();
            callbackContext.success();
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("restartInterstitialInterval", e.toString()));
        }
    }

    private void setAnalyticsCollectionEnabled(final JSONArray args, final CallbackContext callbackContext) {
        try {
            CAS.getSettings().setAnalyticsCollectionEnabled(args.getBoolean(0));
            callbackContext.success();
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("setAnalyticsCollectionEnabled", e.toString()));
        }
    }

    private void setTestDeviceIds(final JSONArray args, final CallbackContext callbackContext) {
        try {
            HashSet<String> testDeviceIds = new HashSet<>();
            for (int i = 0; i < args.length(); i++)
                testDeviceIds.add(args.getString(i));

            CAS.getSettings().setTestDeviceIDs(testDeviceIds);
            callbackContext.success();
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("setTestDeviceIds", e.toString()));
        }
    }

    private void setUserGender(final JSONArray args, final CallbackContext callbackContext) {
        try {
            CAS.getTargetingOptions().setGender(args.getInt(0));
            callbackContext.success();
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("setUserGender", e.toString()));
        }
    }

    private void setUserAge(final JSONArray args, final CallbackContext callbackContext) {
        try {
            CAS.getTargetingOptions().setAge(args.getInt(0));
            callbackContext.success();
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("setUserAge", e.toString()));
        }
    }

    private void getActiveMediationPattern(final CallbackContext callbackContext) {
        try {
            final JSONObject data = new JSONObject();
            data.put("activeMediationPattern", AdNetwork.getActiveNetworkPattern());
            callbackContext.success(data);
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("getActiveMediationPattern", e.toString()));
        }
    }

    private void isActiveMediationNetwork(final JSONArray args, final CallbackContext callbackContext) {
        try {
            final JSONObject data = new JSONObject();
            data.put("isActiveMediationNetwork", AdNetwork.isActiveNetwork(AdNetwork.values()[args.getInt(0)]));
            callbackContext.success(data);
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("isActiveMediationNetwork", e.toString()));
        }
    }

    private void setAllowInterstitialAdsWhenRewardedCostAreLower(final JSONArray args, final CallbackContext callbackContext) {
        try {
            CAS.getSettings().setAllowInterstitialAdsWhenVideoCostAreLower(args.getBoolean(0));
            callbackContext.success();
        } catch (Throwable e) {
            callbackContext.error(debugErrorLog("isActiveMediationNetwork", e.toString()));
        }
    }

    //Utils
    void fireEventTrigger(final String event) {

        final CordovaWebView view = webview;

        cordova.getActivity().runOnUiThread(() -> {

            final String output = "javascript:" +
                    "cordova.fireWindowEvent(" +
                    "'" + event + "'" +
                    ");";

            view.loadUrl(output);
        });
    }

    private String debugErrorLog(final String method, final String exception) {
        return "CAS Cordova java bridge call \"" + method + "\" method failed: " + exception;
    }

    private AdType getAdType(int index) {
        switch (index) {
            case 0:
                return AdType.Banner;
            case 1:
                return AdType.Interstitial;
            case 2:
                return AdType.Rewarded;
            case 3:
                return AdType.Native;
            default:
                return AdType.None;
        }
    }
}
