const CAS_Class = "CASCBridge"

exports.validateIntegration = function (successCallback, errorCallback) {
    cordova.exec(successCallback, errorCallback, CAS_Class, "validateIntegration", []);
}

exports.initialize = function (data, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * {
     *  "managerId": managerId,
     *  "testMode": false
     *  "reward": true,
     *  "interstitial": true,
     *  "banner": true,
     *  "returnAds": true,
     *  "consent": true,
     *  "privacyPolicy":  "https://cleveradssolutions.com"
     * }
     *
     */

    if(data.frameworkName === undefined) {
        data.frameworkName = "Cordova";
        data.frameworkVersion = cordova.version;
    }

    let enableTypes = 0;
    if (data.banner) {
        enableTypes = enableTypes | 1;
    }
    if (data.interstitial) {
        enableTypes = enableTypes | 2;
    }
    if (data.reward) {
        enableTypes = enableTypes | 4;
    }

    delete data['banner'];
    delete data['reward'];
    delete data['interstitial'];

    data.enableAdTypes = enableTypes;

    cordova.exec(successCallback, errorCallback, CAS_Class, "initialize", [data]);
}

exports.initializeWithDefaultParameters = function (data, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * {
     *  "managerId": managerId,
     *  "testMode": false
     *  "reward": true,
     *  "interstitial": true,
     *  "banner": "true",
     *  "returnAds": true,
     *  "debugMode": false,
     *  "isAllowInterstitialAdsWhenRewardedCostAreLower": false,
     *  "validateIntegration": false,
     *  "bannerInSafeArea": true,
     *  "GDPR": 0,
     *  "CCPA": 0,
     *  "taggedAudience": 0,
     *  "loadingMode": 2,
     *  "interstitialInterval": 0,
     *  "refreshBannerDelay": 30,
     *  "analyticsCollectionEnable": false
     *
     * }
     *
     */

    if(data.frameworkName === undefined) {
        data.frameworkName = "Cordova";
        data.frameworkVersion = cordova.version;
    }

    let enableTypes = 0;
    if (data.banner) {
        enableTypes = enableTypes | 1;
    }
    if (data.interstitial) {
        enableTypes = enableTypes | 2;
    }
    if (data.reward) {
        enableTypes = enableTypes | 4;
    }

    delete data['banner'];
    delete data['reward'];
    delete data['interstitial'];

    data.enableAdTypes = enableTypes;

    cordova.exec(successCallback, errorCallback, CAS_Class, "initializeWithDefaultParameters", [data]);
}

exports.setDebugMode = function (isDebug, successCallback, errorCallback) {
    cordova.exec(successCallback, errorCallback, CAS_Class, "setDebugMode", [isDebug]);
}

exports.skipNextReturnAds = function (successCallback, errorCallback) {
    cordova.exec(successCallback, errorCallback, CAS_Class, "skipNextReturnAds", []);
}

exports.loadAd = function (adType, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * Banner = 0,
     * Inter = 1,
     * Reward = 2,
     *
     */

    cordova.exec(successCallback, errorCallback, CAS_Class, "loadAd", [adType]);
}

exports.showAd = function (adType, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * Banner = 0,
     * Inter = 1,
     * Reward = 2,
     *
     */

    cordova.exec(successCallback, errorCallback, CAS_Class, "showAd", [adType]);
}

exports.hideBanner = function (successCallback, errorCallback) {
    cordova.exec(successCallback, errorCallback, CAS_Class, "hideBanner", []);
}

exports.isAdReady = function (adType, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * Banner = 0,
     * Inter = 1,
     * Reward = 2,
     *
     */

    cordova.exec(successCallback, errorCallback, CAS_Class, "isAdReady", [adType]);
}

exports.isEnabled = function (adType, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * Banner = 0,
     * Inter = 1,
     * Reward = 2,
     *
     */

    cordova.exec(successCallback, errorCallback, CAS_Class, "isEnabled", [adType]);
}

exports.enableAd = function (adType, enabled, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * Banner = 0,
     * Inter = 1,
     * Reward = 2,
     *
     */

    cordova.exec(successCallback, errorCallback, CAS_Class, "enableAd", [adType, enabled]);
}

exports.setBannerSizeId = function (sizeId, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * BANNER = 1,
     * AdaptiveBanner = 2,
     * SmartBanner = 3,
     * LEADERBOARD = 4,
     * MEDIUM_RECTANGLE = 5
     *
     */

    cordova.exec(successCallback, errorCallback, CAS_Class, "setBannerSizeId", [sizeId]);
}

exports.setBannerPositionId = function (posId, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * AD_POSITION_TOP_CENTER = 0,
     * AD_POSITION_TOP_LEFT = 1,
     * AD_POSITION_TOP_RIGHT = 2,
     * AD_POSITION_BOTTOM_CENTER = 3,
     * AD_POSITION_BOTTOM_LEFT = 4,
     * AD_POSITION_BOTTOM_RIGHT = 5
     *
     */

    cordova.exec(successCallback, errorCallback, CAS_Class, "setBannerPositionId", [posId]);
}

exports.setLastPageAdContent = function (data, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * {
     *  "headline": "headlineDemo",
     *  "adText": "adTextDemo",
     *  "destinationURL": "destinationUrlExample.com",
     *  "imageURL": "imageUrlDemo",
     *  "iconURL": "imageUrlDemo"
     * }
     *
     */

    cordova.exec(successCallback, errorCallback, CAS_Class, "setLastPageAdContent", [data]);
}

exports.setBannerInSafeArea = function (isEnabled, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * IsEnabled = true
     *
     */

    cordova.exec(successCallback, errorCallback, CAS_Class, "setBannerInSafeArea", [isEnabled]);
}

exports.getSDKVersion = function (successCallback, errorCallback) {
    cordova.exec(successCallback, errorCallback, CAS_Class, "getSDKVersion", []);
}

exports.setUserConsentStatus = function (isAccept, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * Undefined = 0, Denied = 1, Accepted = 2
     *
     */

    cordova.exec(successCallback, errorCallback, CAS_Class, "setUserConsentStatus", [isAccept]);
}

exports.setDoNotSellStatus = function (isAccept, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * Undefined = 0, OPT_OUT_SALE = 1, OPT_IN_SALE = 2
     *
     */

    cordova.exec(successCallback, errorCallback, CAS_Class, "setDoNotSellStatus", [isAccept]);
}

exports.setTaggedAudience = function (isAccept, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * Undefined = 0, CHILDREN = 1, NOT_CHILDREN = 2
     *
     */

    cordova.exec(successCallback, errorCallback, CAS_Class, "setTaggedAudience", [isAccept]);
}

exports.setMutedAdSounds = function (isEnable, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * On = 1, Off = 0
     *
     */

    cordova.exec(successCallback, errorCallback, CAS_Class, "setMutedAdSounds", [isEnable]);
}

exports.setLoadingMode = function (loadingMode, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * FastestRequests = 0,
     * FastRequests = 1,
     * Optimal (Default) = 2,
     * HighPerformance = 3,
     * HighestPerformance = 4,
     * Manual = 5
     *
     */

    cordova.exec(successCallback, errorCallback, CAS_Class, "setLoadingMode", [loadingMode]);
}

exports.setRefreshBannerDelay = function (delay, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * Delay = 30
     *
     */

    cordova.exec(successCallback, errorCallback, CAS_Class, "setRefreshBannerDelay", [delay]);
}

exports.setInterstitialInterval = function (interval, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * Interval = 30
     *
     */

    cordova.exec(successCallback, errorCallback, CAS_Class, "setInterstitialInterval", [interval]);
}

exports.restartInterstitialInterval = function (successCallback, errorCallback) {
    cordova.exec(successCallback, errorCallback, CAS_Class, "restartInterstitialInterval", []);
}

exports.setAnalyticsCollectionEnabled = function (isEnabled, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * isEnabled = true
     *
     */

    cordova.exec(successCallback, errorCallback, CAS_Class, "setAnalyticsCollectionEnabled", [isEnabled]);
}

exports.setTestDeviceIds = function (testIds, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * massive of string test ids
     *
     */

    cordova.exec(successCallback, errorCallback, CAS_Class, "setAnalyticsCollectionEnabled", testIds);
}

exports.setUserGender = function (gender, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * Unknown Gender = 0, Male Gender = 1, Female Gender = 2
     *
     */

    cordova.exec(successCallback, errorCallback, CAS_Class, "setUserGender", [gender]);
}

exports.setUserAge = function (userAge, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * Set user age. Limitation: 1-99 and 0 is 'unknown'
     *
     */

    cordova.exec(successCallback, errorCallback, CAS_Class, "setUserAge", [userAge]);
}

exports.getActiveMediationPattern = function (successCallback, errorCallback) {
    cordova.exec(successCallback, errorCallback, CAS_Class, "getActiveMediationPattern", []);
}

exports.isActiveMediationNetwork = function (networkId, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * Id of Network
     *
     */

    cordova.exec(successCallback, errorCallback, CAS_Class, "isActiveMediationNetwork", [networkId]);
}

exports.setAllowInterstitialAdsWhenRewardedCostAreLower = function (isAllow, successCallback, errorCallback) {
    /**
     * Data Format ::
     *
     * is Allow
     *
     */

    cordova.exec(successCallback, errorCallback, CAS_Class, "setAllowInterstitialAdsWhenRewardedCostAreLower", [isAllow]);
}




